"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const util_1 = require("../util");
const config_1 = require("./../config");
const inquirer = require("inquirer");
const path = require("path");
const chalk = require("chalk");
const fs = require("fs-extra");
const generators_1 = require("../generators");
class ComponentsCommand extends command_1.Command {
    doExec(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let command = args._[1];
            if (command == "add")
                return this.add();
            else if (command == "remove")
                return this.remove();
            else if (command == "list")
                return this.list();
            else
                return this.printHelp();
        });
    }
    validateKey(config, key) {
        if (!key)
            return "Missing key";
        if (!/^[a-zA-Z][a-zA-Z0-9]*$/.exec(key))
            return "Invalid key! Only characters and numbers are allowed!";
        let existing = config.components.find(i => i.key == key);
        if (existing)
            return "A component with name '" + key + "' already exists!";
        return true;
    }
    add() {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Adding new component");
            let config = yield config_1.getConfig();
            let answers = yield inquirer.prompt([
                { type: 'input', name: 'key', message: 'Key', required: true, validate: (i) => this.validateKey(config, i) },
                { type: 'list', name: 'type', message: 'Component type', required: true, choices: Object.keys(config_1.ComponentType) }
            ]);
            const componentPath = path.resolve(config.dirs.src, answers.key);
            if (fs.existsSync(componentPath))
                return util_1.logError("Path " + componentPath + " already exists!");
            util_1.logDebug("Creating component dir " + chalk.cyan(componentPath));
            yield fs.ensureDir(componentPath);
            if (!config.components)
                config.components = [];
            let component = { key: answers.key, type: answers.type };
            if (answers.type == "typescript") {
                component.script = "app.js";
                yield this.prepareTypescript(config, component, componentPath);
            }
            else if (answers.type == "angular") {
                yield this.prepareAngular(config, component, componentPath);
            }
            else if (answers.type == "rollup") {
                yield this.prepareRollup(config, component, componentPath);
            }
            else {
            }
            config.components.push(component);
            yield config_1.setConfig(config);
        });
    }
    prepareTypescript(config, component, componentPath) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Setting up a " + chalk.cyan("typescript") + " project at " + chalk.magenta(path.relative(process.cwd(), componentPath)));
            let pkgData = yield generators_1.generateFile("package.json", { name: config.name, component: component.key });
            fs.writeFileSync(path.join(componentPath, "package.json"), pkgData);
            let tsData = yield generators_1.generateFile("typescript_tsconfig.json", {});
            fs.writeFileSync(path.join(componentPath, "tsconfig.json"), tsData);
            yield fs.ensureDir(path.join(componentPath, "src"));
            let appData = yield generators_1.generateFile("typescript_app.ts", {});
            fs.writeFileSync(path.join(componentPath, "src", "app.ts"), appData);
            yield util_1.runCommand("npm", ["install", "@types/node", "--save-dev"], componentPath);
            yield fs.ensureDir(path.join(componentPath, "src"));
        });
    }
    prepareAngular(config, component, componentPath) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Setting up an " + chalk.cyan("angular") + " project at " + chalk.magenta(path.relative(process.cwd(), componentPath)));
            yield util_1.runCommand("npm", ["i", "@angular/cli", "--save-dev"], process.cwd());
            let ngCommand = yield util_1.findBinCommand("ng");
            yield util_1.runCommand(ngCommand, ["new", component.key, "--directory=.", "--interactive=false", "--minimal=true", "--skipGit=true", "--skipTests=true", "--style=scss"], componentPath);
        });
    }
    prepareRollup(config, component, componentPath) {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Setting up a " + chalk.cyan("rollup") + " project at " + chalk.magenta(path.relative(process.cwd(), componentPath)));
            let rcData = yield generators_1.generateFile("rollup_rollup.config.js", { name: config.name + "." + component.key });
            fs.writeFileSync(path.join(componentPath, "rollup.config.js"), rcData);
            let pkgData = yield generators_1.generateFile("package.json", { name: config.name, component: component.key });
            fs.writeFileSync(path.join(componentPath, "package.json"), pkgData);
            yield util_1.runCommand("npm", ["i", "rollup", "--save-dev"]);
            yield util_1.runCommand("npm", ["i", "rollup-plugin-typescript2", "--save-dev"]);
            let tsData = yield generators_1.generateFile("rollup_tsconfig.json", {});
            fs.writeFileSync(path.join(componentPath, "tsconfig.json"), tsData);
            fs.ensureDirSync(path.join(componentPath, "src"));
            let indexData = yield generators_1.generateFile("rollup_index.ts", { name: "@" + config.name + "/" + component.key });
            fs.writeFileSync(path.join(componentPath, "src", "index.ts"), indexData);
        });
    }
    remove() {
        return __awaiter(this, void 0, void 0, function* () {
            util_1.log("Removing component");
            let config = yield config_1.getConfig();
            let options = config.components.map(i => i.key);
            if (options.length == 0)
                return util_1.log("No components to remove");
            let srcRoot = path.resolve(config.dirs.src);
            let answers = yield inquirer.prompt([
                { type: 'list', name: 'component', message: 'Which component would you like to remove?', required: true, choices: options },
                { type: 'list', name: 'src', message: 'Remove source files at path ' + chalk.magenta(path.relative(process.cwd(), srcRoot)) + '?', required: true, choices: ['no', 'yes'] }
            ]);
            config.components.splice(config.components.findIndex(i => i.key == answers.component), 1);
            yield config_1.setConfig(config);
            if (answers.src == 'yes') {
                util_1.logDebug("Unlinking component at " + chalk.magenta(path.relative(process.cwd(), srcRoot)));
                yield fs.remove(path.join(srcRoot, answers.component));
            }
        });
    }
    list() {
        return __awaiter(this, void 0, void 0, function* () {
            let config = yield config_1.getConfig();
            if (!config)
                return util_1.logError("No project exists at " + process.cwd());
            for (let i = 1; i <= config.components.length; i++) {
                let comp = config.components[i - 1];
                console.log(chalk.yellow(i + ". " + comp.key + " (" + chalk.white(comp.type) + ")"));
            }
        });
    }
    printHelp() {
        console.log(`
Synopsis

    spast components <command> [options]

Manage project components

    add ...................... Add a new component and save into spast.json
    remove ................... Remove a component and update spast.json
    list ..................... List all components from spast.json
    `);
    }
}
exports.ComponentsCommand = ComponentsCommand;
